-module(glyph@internal@decoders).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([gateway_hello_decoder/1, gateway_partial_app_decoder/0, user_decoder/0, message_decoder/1, member_decoder/0, team_decoder/0, install_params_decoder/0, application_decoder/0, session_start_limit_decoder/0, get_gateway_bot_decoder/0, gateway_event_decoder/0, gateway_ready_decoder/1]).

-spec gateway_hello_decoder(gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:hello_event()} |
    {error, list(gleam@dynamic:decode_error())}.
gateway_hello_decoder(Dyn) ->
    (gleam@dynamic:decode1(
        fun(Field@0) -> {hello_event, Field@0} end,
        gleam@dynamic:field(
            <<"heartbeat_interval"/utf8>>,
            fun gleam@dynamic:int/1
        )
    ))(Dyn).

-spec gateway_partial_app_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:ready_application()} |
    {error, list(gleam@dynamic:decode_error())}).
gateway_partial_app_decoder() ->
    gleam@dynamic:decode2(
        fun(Field@0, Field@1) -> {ready_application, Field@0, Field@1} end,
        gleam@dynamic:field(<<"id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"flags"/utf8>>, fun gleam@dynamic:int/1)
    ).

-spec all_errors({ok, any()} | {error, list(gleam@dynamic:decode_error())}) -> list(gleam@dynamic:decode_error()).
all_errors(Result) ->
    case Result of
        {ok, _} ->
            [];

        {error, Errors} ->
            Errors
    end.

-spec application(
    fun((LHE, LHF, LHG, LHH, LHI, LHJ, LHK, LHL, LHM, LHN, LHO, LHP, LHQ, LHR, LHS, LHT, LHU, LHV, LHW, LHX, LHY, LHZ, LIA, LIB, LIC, LID) -> glyph@models@discord:application()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHE} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHF} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHG} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHH} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHI} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHJ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHK} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHL} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHM} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHN} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHO} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHP} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHQ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHR} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHS} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHT} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHU} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHV} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHW} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHX} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHY} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LHZ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LIA} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LIB} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LIC} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LID} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok, glyph@models@discord:application()} |
    {error, list(gleam@dynamic:decode_error())}).
application(
    Constructor,
    Id,
    Name,
    Icon,
    Description,
    Rpc_origins,
    Bot_public,
    Bot_require_code_grant,
    Bot,
    Terms_of_service_url,
    Privacy_policy_url,
    Owner,
    Summary,
    Verify_key,
    Team,
    Guild_id,
    Primary_sku_id,
    Slug,
    Cover_image,
    Flags,
    Approximate_guild_count,
    Redirect_uris,
    Interactions_endpoint_url,
    Role_connections_verification_url,
    Tags,
    Install_params,
    Custom_install_url
) ->
    fun(App) ->
        case {Id(App),
            Name(App),
            Icon(App),
            Description(App),
            Rpc_origins(App),
            Bot_public(App),
            Bot_require_code_grant(App),
            Bot(App),
            Terms_of_service_url(App),
            Privacy_policy_url(App),
            Owner(App),
            Summary(App),
            Verify_key(App),
            Team(App),
            Guild_id(App),
            Primary_sku_id(App),
            Slug(App),
            Cover_image(App),
            Flags(App),
            Approximate_guild_count(App),
            Redirect_uris(App),
            Interactions_endpoint_url(App),
            Role_connections_verification_url(App),
            Tags(App),
            Install_params(App),
            Custom_install_url(App)} of
            {{ok, A},
                {ok, B},
                {ok, C},
                {ok, D},
                {ok, E},
                {ok, F},
                {ok, G},
                {ok, H},
                {ok, I},
                {ok, J},
                {ok, K},
                {ok, L},
                {ok, M},
                {ok, N},
                {ok, O},
                {ok, P},
                {ok, Q},
                {ok, R},
                {ok, S},
                {ok, T},
                {ok, U},
                {ok, V},
                {ok, W},
                {ok, X},
                {ok, Y},
                {ok, Z}} ->
                {ok,
                    Constructor(
                        A,
                        B,
                        C,
                        D,
                        E,
                        F,
                        G,
                        H,
                        I,
                        J,
                        K,
                        L,
                        M,
                        N,
                        O,
                        P,
                        Q,
                        R,
                        S,
                        T,
                        U,
                        V,
                        W,
                        X,
                        Y,
                        Z
                    )};

            {A@1,
                B@1,
                C@1,
                D@1,
                E@1,
                F@1,
                G@1,
                H@1,
                I@1,
                J@1,
                K@1,
                L@1,
                M@1,
                N@1,
                O@1,
                P@1,
                Q@1,
                R@1,
                S@1,
                T@1,
                U@1,
                V@1,
                W@1,
                X@1,
                Y@1,
                Z@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1),
                            all_errors(B@1),
                            all_errors(C@1),
                            all_errors(D@1),
                            all_errors(E@1),
                            all_errors(F@1),
                            all_errors(G@1),
                            all_errors(H@1),
                            all_errors(I@1),
                            all_errors(J@1),
                            all_errors(K@1),
                            all_errors(L@1),
                            all_errors(M@1),
                            all_errors(N@1),
                            all_errors(O@1),
                            all_errors(P@1),
                            all_errors(Q@1),
                            all_errors(R@1),
                            all_errors(S@1),
                            all_errors(T@1),
                            all_errors(U@1),
                            all_errors(V@1),
                            all_errors(W@1),
                            all_errors(X@1),
                            all_errors(Y@1),
                            all_errors(Z@1)]
                    )}
        end
    end.

-spec user(
    fun((LJG, LJH, LJI, LJJ, LJK, LJL, LJM, LJN, LJO, LJP, LJQ, LJR, LJS, LJT, LJU, LJV) -> glyph@models@discord:user()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJG} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJH} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJI} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJJ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJK} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJL} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJM} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJN} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJO} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJP} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJQ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJR} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJS} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJT} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJU} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LJV} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok, glyph@models@discord:user()} |
    {error, list(gleam@dynamic:decode_error())}).
user(
    Constructor,
    Id,
    Username,
    Discriminator,
    Global_name,
    Avatar,
    Bot,
    System,
    Mfa_enabled,
    Banner,
    Accent_color,
    Locale,
    Email,
    Flags,
    Premium_type,
    Public_flags,
    Avatar_decoration
) ->
    fun(User) ->
        case {Id(User),
            Username(User),
            Discriminator(User),
            Global_name(User),
            Avatar(User),
            Bot(User),
            System(User),
            Mfa_enabled(User),
            Banner(User),
            Accent_color(User),
            Locale(User),
            Email(User),
            Flags(User),
            Premium_type(User),
            Public_flags(User),
            Avatar_decoration(User)} of
            {{ok, A},
                {ok, B},
                {ok, C},
                {ok, D},
                {ok, E},
                {ok, F},
                {ok, G},
                {ok, H},
                {ok, I},
                {ok, J},
                {ok, K},
                {ok, L},
                {ok, M},
                {ok, N},
                {ok, O},
                {ok, P}} ->
                {ok,
                    Constructor(A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P)};

            {A@1,
                B@1,
                C@1,
                D@1,
                E@1,
                F@1,
                G@1,
                H@1,
                I@1,
                J@1,
                K@1,
                L@1,
                M@1,
                N@1,
                O@1,
                P@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1),
                            all_errors(B@1),
                            all_errors(C@1),
                            all_errors(D@1),
                            all_errors(E@1),
                            all_errors(F@1),
                            all_errors(G@1),
                            all_errors(H@1),
                            all_errors(I@1),
                            all_errors(J@1),
                            all_errors(K@1),
                            all_errors(L@1),
                            all_errors(M@1),
                            all_errors(N@1),
                            all_errors(O@1),
                            all_errors(P@1)]
                    )}
        end
    end.

-spec user_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:user()} |
    {error, list(gleam@dynamic:decode_error())}).
user_decoder() ->
    user(
        fun(Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6, Field@7, Field@8, Field@9, Field@10, Field@11, Field@12, Field@13, Field@14, Field@15) -> {user, Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6, Field@7, Field@8, Field@9, Field@10, Field@11, Field@12, Field@13, Field@14, Field@15} end,
        gleam@dynamic:field(<<"id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"username"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(
            <<"discriminator"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"global_name"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"avatar"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(<<"bot"/utf8>>, fun gleam@dynamic:bool/1),
        gleam@dynamic:optional_field(
            <<"system"/utf8>>,
            fun gleam@dynamic:bool/1
        ),
        gleam@dynamic:optional_field(
            <<"mfa_enabled"/utf8>>,
            fun gleam@dynamic:bool/1
        ),
        gleam@dynamic:optional_field(
            <<"banner"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"accent_color"/utf8>>,
            fun gleam@dynamic:int/1
        ),
        gleam@dynamic:optional_field(
            <<"locale"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"email"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(<<"flags"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:optional_field(
            <<"premium_type"/utf8>>,
            fun gleam@dynamic:int/1
        ),
        gleam@dynamic:optional_field(
            <<"public_flags"/utf8>>,
            fun gleam@dynamic:int/1
        ),
        gleam@dynamic:optional_field(
            <<"avatar_decoration"/utf8>>,
            fun gleam@dynamic:string/1
        )
    ).

-spec message(
    fun((LKO, LKP, LKQ, LKR, LKS, LKT, LKU, LKV) -> glyph@models@discord:message()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKO} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKP} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKQ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKR} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKS} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKT} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKU} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LKV} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok, glyph@models@discord:message()} |
    {error, list(gleam@dynamic:decode_error())}).
message(
    Constructor,
    Id,
    Channel_id,
    Author,
    Content,
    Tts,
    Mention_everyone,
    Pinned,
    Message_type
) ->
    fun(Message) ->
        case {Id(Message),
            Channel_id(Message),
            Author(Message),
            Content(Message),
            Tts(Message),
            Mention_everyone(Message),
            Pinned(Message),
            Message_type(Message)} of
            {{ok, A},
                {ok, B},
                {ok, C},
                {ok, D},
                {ok, E},
                {ok, F},
                {ok, G},
                {ok, H}} ->
                {ok, Constructor(A, B, C, D, E, F, G, H)};

            {A@1, B@1, C@1, D@1, E@1, F@1, G@1, H@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1),
                            all_errors(B@1),
                            all_errors(C@1),
                            all_errors(D@1),
                            all_errors(E@1),
                            all_errors(F@1),
                            all_errors(G@1),
                            all_errors(H@1)]
                    )}
        end
    end.

-spec message_decoder(gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:message()} |
    {error, list(gleam@dynamic:decode_error())}.
message_decoder(Dyn) ->
    (message(
        fun(Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6, Field@7) -> {message, Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6, Field@7} end,
        gleam@dynamic:field(<<"id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"channel_id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"author"/utf8>>, user_decoder()),
        gleam@dynamic:field(<<"content"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"tts"/utf8>>, fun gleam@dynamic:bool/1),
        gleam@dynamic:field(
            <<"mention_everyone"/utf8>>,
            fun gleam@dynamic:bool/1
        ),
        gleam@dynamic:field(<<"pinned"/utf8>>, fun gleam@dynamic:bool/1),
        gleam@dynamic:field(<<"type"/utf8>>, fun gleam@dynamic:int/1)
    ))(Dyn).

-spec team(
    fun((LLH, LLI, LLJ, LLK, LLL) -> glyph@models@discord:team()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLH} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLI} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLJ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLK} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLL} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok, glyph@models@discord:team()} |
    {error, list(gleam@dynamic:decode_error())}).
team(Constructor, Id, Icon, Members, Name, Owner_user_id) ->
    fun(Team) ->
        case {Id(Team),
            Icon(Team),
            Members(Team),
            Name(Team),
            Owner_user_id(Team)} of
            {{ok, A}, {ok, B}, {ok, C}, {ok, D}, {ok, E}} ->
                {ok, Constructor(A, B, C, D, E)};

            {A@1, B@1, C@1, D@1, E@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1),
                            all_errors(B@1),
                            all_errors(C@1),
                            all_errors(D@1),
                            all_errors(E@1)]
                    )}
        end
    end.

-spec member(
    fun((LLT, LLU, LLV, LLW) -> glyph@models@discord:member()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLT} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLU} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLV} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LLW} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok, glyph@models@discord:member()} |
    {error, list(gleam@dynamic:decode_error())}).
member(Constructor, Membership_state, Team_id, User, Role) ->
    fun(Member) ->
        case {Membership_state(Member),
            Team_id(Member),
            User(Member),
            Role(Member)} of
            {{ok, A}, {ok, B}, {ok, C}, {ok, D}} ->
                {ok, Constructor(A, B, C, D)};

            {A@1, B@1, C@1, D@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1),
                            all_errors(B@1),
                            all_errors(C@1),
                            all_errors(D@1)]
                    )}
        end
    end.

-spec member_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:member()} |
    {error, list(gleam@dynamic:decode_error())}).
member_decoder() ->
    member(
        fun(Field@0, Field@1, Field@2, Field@3) -> {member, Field@0, Field@1, Field@2, Field@3} end,
        gleam@dynamic:field(
            <<"membership_state"/utf8>>,
            fun gleam@dynamic:int/1
        ),
        gleam@dynamic:field(<<"team_id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"user"/utf8>>, user_decoder()),
        gleam@dynamic:field(<<"role"/utf8>>, fun gleam@dynamic:string/1)
    ).

-spec team_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:team()} |
    {error, list(gleam@dynamic:decode_error())}).
team_decoder() ->
    team(
        fun(Field@0, Field@1, Field@2, Field@3, Field@4) -> {team, Field@0, Field@1, Field@2, Field@3, Field@4} end,
        gleam@dynamic:field(<<"id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:optional_field(
            <<"icon"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:field(
            <<"members"/utf8>>,
            gleam@dynamic:list(member_decoder())
        ),
        gleam@dynamic:field(<<"name"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(
            <<"owner_user_id"/utf8>>,
            fun gleam@dynamic:string/1
        )
    ).

-spec install_params(
    fun((LMD, LME) -> glyph@models@discord:install_params()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMD} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LME} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:install_params()} |
    {error, list(gleam@dynamic:decode_error())}).
install_params(Constructor, Scopes, Permissions) ->
    fun(Install_params) ->
        case {Scopes(Install_params), Permissions(Install_params)} of
            {{ok, A}, {ok, B}} ->
                {ok, Constructor(A, B)};

            {A@1, B@1} ->
                {error, gleam@list:concat([all_errors(A@1), all_errors(B@1)])}
        end
    end.

-spec install_params_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:install_params()} |
    {error, list(gleam@dynamic:decode_error())}).
install_params_decoder() ->
    install_params(
        fun(Field@0, Field@1) -> {install_params, Field@0, Field@1} end,
        gleam@dynamic:field(
            <<"scopes"/utf8>>,
            gleam@dynamic:list(fun gleam@dynamic:string/1)
        ),
        gleam@dynamic:field(<<"permissions"/utf8>>, fun gleam@dynamic:string/1)
    ).

-spec application_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:application()} |
    {error, list(gleam@dynamic:decode_error())}).
application_decoder() ->
    application(
        fun(Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6, Field@7, Field@8, Field@9, Field@10, Field@11, Field@12, Field@13, Field@14, Field@15, Field@16, Field@17, Field@18, Field@19, Field@20, Field@21, Field@22, Field@23, Field@24, Field@25) -> {application, Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6, Field@7, Field@8, Field@9, Field@10, Field@11, Field@12, Field@13, Field@14, Field@15, Field@16, Field@17, Field@18, Field@19, Field@20, Field@21, Field@22, Field@23, Field@24, Field@25} end,
        gleam@dynamic:field(<<"id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"name"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:optional_field(
            <<"icon"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:field(<<"description"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:optional_field(
            <<"rpc_origins"/utf8>>,
            gleam@dynamic:list(fun gleam@dynamic:string/1)
        ),
        gleam@dynamic:field(<<"bot_public"/utf8>>, fun gleam@dynamic:bool/1),
        gleam@dynamic:field(
            <<"bot_require_code_grant"/utf8>>,
            fun gleam@dynamic:bool/1
        ),
        gleam@dynamic:optional_field(<<"bot"/utf8>>, user_decoder()),
        gleam@dynamic:optional_field(
            <<"terms_of_service_url"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"privacy_policy_url"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(<<"owner"/utf8>>, user_decoder()),
        gleam@dynamic:optional_field(
            <<"summary"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:field(<<"verify_key"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:optional_field(<<"team"/utf8>>, team_decoder()),
        gleam@dynamic:optional_field(
            <<"guild_id"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"primary_sku_id"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"slug"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"cover_image"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(<<"flags"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:optional_field(
            <<"approximate_guild_count"/utf8>>,
            fun gleam@dynamic:int/1
        ),
        gleam@dynamic:optional_field(
            <<"redirect_uris"/utf8>>,
            gleam@dynamic:list(fun gleam@dynamic:string/1)
        ),
        gleam@dynamic:optional_field(
            <<"interactions_endpoint_url"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"role_connections_verification_url"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"tags"/utf8>>,
            gleam@dynamic:list(fun gleam@dynamic:string/1)
        ),
        gleam@dynamic:optional_field(
            <<"install_params"/utf8>>,
            install_params_decoder()
        ),
        gleam@dynamic:optional_field(
            <<"custom_install_url"/utf8>>,
            fun gleam@dynamic:string/1
        )
    ).

-spec get_gateway_bot(
    fun((LMJ, LMK, LML) -> glyph@models@discord:get_gateway_bot()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMJ} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMK} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LML} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:get_gateway_bot()} |
    {error, list(gleam@dynamic:decode_error())}).
get_gateway_bot(Constructor, Url, Shards, Session_start_limit) ->
    fun(Gateway_bot) ->
        case {Url(Gateway_bot),
            Shards(Gateway_bot),
            Session_start_limit(Gateway_bot)} of
            {{ok, A}, {ok, B}, {ok, C}} ->
                {ok, Constructor(A, B, C)};

            {A@1, B@1, C@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1), all_errors(B@1), all_errors(C@1)]
                    )}
        end
    end.

-spec session_start_limit(
    fun((LMR, LMS, LMT, LMU) -> glyph@models@discord:session_start_limit()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMR} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMS} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMT} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LMU} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:session_start_limit()} |
    {error, list(gleam@dynamic:decode_error())}).
session_start_limit(Constructor, Total, Remaining, Reset_after, Max_concurrency) ->
    fun(Session_start_limit) ->
        case {Total(Session_start_limit),
            Remaining(Session_start_limit),
            Reset_after(Session_start_limit),
            Max_concurrency(Session_start_limit)} of
            {{ok, A}, {ok, B}, {ok, C}, {ok, D}} ->
                {ok, Constructor(A, B, C, D)};

            {A@1, B@1, C@1, D@1} ->
                {error,
                    gleam@list:concat(
                        [all_errors(A@1),
                            all_errors(B@1),
                            all_errors(C@1),
                            all_errors(D@1)]
                    )}
        end
    end.

-spec session_start_limit_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:session_start_limit()} |
    {error, list(gleam@dynamic:decode_error())}).
session_start_limit_decoder() ->
    session_start_limit(
        fun(Field@0, Field@1, Field@2, Field@3) -> {session_start_limit, Field@0, Field@1, Field@2, Field@3} end,
        gleam@dynamic:field(<<"total"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"remaining"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"reset_after"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"max_concurrency"/utf8>>, fun gleam@dynamic:int/1)
    ).

-spec get_gateway_bot_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:get_gateway_bot()} |
    {error, list(gleam@dynamic:decode_error())}).
get_gateway_bot_decoder() ->
    get_gateway_bot(
        fun(Field@0, Field@1, Field@2) -> {get_gateway_bot, Field@0, Field@1, Field@2} end,
        gleam@dynamic:field(<<"url"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(<<"shards"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(
            <<"session_start_limit"/utf8>>,
            session_start_limit_decoder()
        )
    ).

-spec gateway_event(
    fun((LNB, LNC, LND, LNE) -> glyph@models@discord:gateway_event()),
    fun((gleam@dynamic:dynamic_()) -> {ok, LNB} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LNC} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LND} |
        {error, list(gleam@dynamic:decode_error())}),
    fun((gleam@dynamic:dynamic_()) -> {ok, LNE} |
        {error, list(gleam@dynamic:decode_error())})
) -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:gateway_event()} |
    {error, list(gleam@dynamic:decode_error())}).
gateway_event(Constructor, Op, D, S, T) ->
    fun(Gateway_event) ->
        case {Op(Gateway_event),
            D(Gateway_event),
            S(Gateway_event),
            T(Gateway_event)} of
            {{ok, Op@1}, {ok, D@1}, {ok, S@1}, {ok, T@1}} ->
                {ok, Constructor(Op@1, D@1, S@1, T@1)};

            {Op@2, D@2, S@2, T@2} ->
                {error,
                    gleam@list:concat(
                        [all_errors(Op@2),
                            all_errors(D@2),
                            all_errors(S@2),
                            all_errors(T@2)]
                    )}
        end
    end.

-spec gateway_event_decoder() -> fun((gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:gateway_event()} |
    {error, list(gleam@dynamic:decode_error())}).
gateway_event_decoder() ->
    gateway_event(
        fun(Field@0, Field@1, Field@2, Field@3) -> {gateway_event, Field@0, Field@1, Field@2, Field@3} end,
        gleam@dynamic:field(<<"op"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"d"/utf8>>, fun gleam@dynamic:dynamic/1),
        gleam@dynamic:optional_field(<<"s"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:optional_field(<<"t"/utf8>>, fun gleam@dynamic:string/1)
    ).

-spec gateway_ready_decoder(gleam@dynamic:dynamic_()) -> {ok,
        glyph@models@discord:ready_event()} |
    {error, list(gleam@dynamic:decode_error())}.
gateway_ready_decoder(Dyn) ->
    (gleam@dynamic:decode7(
        fun(Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6) -> {ready_event, Field@0, Field@1, Field@2, Field@3, Field@4, Field@5, Field@6} end,
        gleam@dynamic:field(<<"v"/utf8>>, fun gleam@dynamic:int/1),
        gleam@dynamic:field(<<"user"/utf8>>, user_decoder()),
        gleam@dynamic:field(<<"guilds"/utf8>>, fun gleam@dynamic:dynamic/1),
        gleam@dynamic:field(<<"session_id"/utf8>>, fun gleam@dynamic:string/1),
        gleam@dynamic:field(
            <<"resume_gateway_url"/utf8>>,
            fun gleam@dynamic:string/1
        ),
        gleam@dynamic:optional_field(
            <<"shard"/utf8>>,
            gleam@dynamic:list(fun gleam@dynamic:int/1)
        ),
        gleam@dynamic:field(
            <<"application"/utf8>>,
            gateway_partial_app_decoder()
        )
    ))(Dyn).
