-module(glyph@internal@encoders).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([message_to_json/1]).

-spec message_reference_to_json(glyph@models@discord:message_reference()) -> gleam@json:json().
message_reference_to_json(Ref) ->
    gleam@json:object(
        [{<<"message_id"/utf8>>,
                gleam@json:nullable(
                    erlang:element(2, Ref),
                    fun gleam@json:string/1
                )},
            {<<"channel_id"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Ref),
                    fun gleam@json:string/1
                )},
            {<<"guild_id"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Ref),
                    fun gleam@json:string/1
                )},
            {<<"fail_if_not_exists"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Ref),
                    fun gleam@json:bool/1
                )}]
    ).

-spec allowed_mentions_to_json(glyph@models@discord:allowed_mentions()) -> gleam@json:json().
allowed_mentions_to_json(Ment) ->
    gleam@json:object(
        [{<<"parse"/utf8>>,
                gleam@json:array(
                    gleam@list:map(
                        erlang:element(2, Ment),
                        fun glyph@models@discord:mention_type_to_string/1
                    ),
                    fun gleam@json:string/1
                )},
            {<<"roles"/utf8>>,
                gleam@json:array(
                    erlang:element(3, Ment),
                    fun gleam@json:string/1
                )},
            {<<"users"/utf8>>,
                gleam@json:array(
                    erlang:element(4, Ment),
                    fun gleam@json:string/1
                )},
            {<<"replied_user"/utf8>>, gleam@json:bool(erlang:element(5, Ment))}]
    ).

-spec embed_footer_to_json(glyph@models@discord:embed_footer()) -> gleam@json:json().
embed_footer_to_json(Footer) ->
    gleam@json:object(
        [{<<"text"/utf8>>, gleam@json:string(erlang:element(2, Footer))},
            {<<"icon_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Footer),
                    fun gleam@json:string/1
                )},
            {<<"proxy_icon_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Footer),
                    fun gleam@json:string/1
                )}]
    ).

-spec embed_img_to_json(glyph@models@discord:embed_image()) -> gleam@json:json().
embed_img_to_json(Img) ->
    gleam@json:object(
        [{<<"url"/utf8>>, gleam@json:string(erlang:element(2, Img))},
            {<<"proxy_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Img),
                    fun gleam@json:string/1
                )},
            {<<"height"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Img),
                    fun gleam@json:int/1
                )},
            {<<"width"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Img),
                    fun gleam@json:int/1
                )}]
    ).

-spec embed_thumbnail_to_json(glyph@models@discord:embed_thumbnail()) -> gleam@json:json().
embed_thumbnail_to_json(Img) ->
    gleam@json:object(
        [{<<"url"/utf8>>, gleam@json:string(erlang:element(2, Img))},
            {<<"proxy_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Img),
                    fun gleam@json:string/1
                )},
            {<<"height"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Img),
                    fun gleam@json:int/1
                )},
            {<<"width"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Img),
                    fun gleam@json:int/1
                )}]
    ).

-spec embed_video_to_json(glyph@models@discord:embed_video()) -> gleam@json:json().
embed_video_to_json(Vid) ->
    gleam@json:object(
        [{<<"url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(2, Vid),
                    fun gleam@json:string/1
                )},
            {<<"proxy_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Vid),
                    fun gleam@json:string/1
                )},
            {<<"height"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Vid),
                    fun gleam@json:int/1
                )},
            {<<"width"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Vid),
                    fun gleam@json:int/1
                )}]
    ).

-spec embed_provider_to_json(glyph@models@discord:embed_provider()) -> gleam@json:json().
embed_provider_to_json(Prov) ->
    gleam@json:object(
        [{<<"name"/utf8>>,
                gleam@json:nullable(
                    erlang:element(2, Prov),
                    fun gleam@json:string/1
                )},
            {<<"url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Prov),
                    fun gleam@json:string/1
                )}]
    ).

-spec embed_author_to_json(glyph@models@discord:embed_author()) -> gleam@json:json().
embed_author_to_json(Author) ->
    gleam@json:object(
        [{<<"name"/utf8>>, gleam@json:string(erlang:element(2, Author))},
            {<<"url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Author),
                    fun gleam@json:string/1
                )},
            {<<"icon_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Author),
                    fun gleam@json:string/1
                )},
            {<<"proxy_icon_url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Author),
                    fun gleam@json:string/1
                )}]
    ).

-spec embed_field_to_json(glyph@models@discord:embed_field()) -> gleam@json:json().
embed_field_to_json(Field) ->
    gleam@json:object(
        [{<<"name"/utf8>>, gleam@json:string(erlang:element(2, Field))},
            {<<"value"/utf8>>, gleam@json:string(erlang:element(3, Field))},
            {<<"inline"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Field),
                    fun gleam@json:bool/1
                )}]
    ).

-spec embed_to_json(glyph@models@discord:embed()) -> gleam@json:json().
embed_to_json(Embed) ->
    gleam@json:object(
        [{<<"title"/utf8>>,
                gleam@json:nullable(
                    erlang:element(2, Embed),
                    fun gleam@json:string/1
                )},
            {<<"description"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Embed),
                    fun gleam@json:string/1
                )},
            {<<"url"/utf8>>,
                gleam@json:nullable(
                    erlang:element(4, Embed),
                    fun gleam@json:string/1
                )},
            {<<"timestamp"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Embed),
                    fun gleam@json:string/1
                )},
            {<<"color"/utf8>>,
                gleam@json:nullable(
                    erlang:element(6, Embed),
                    fun gleam@json:int/1
                )},
            {<<"footer"/utf8>>,
                gleam@json:nullable(
                    erlang:element(7, Embed),
                    fun embed_footer_to_json/1
                )},
            {<<"image"/utf8>>,
                gleam@json:nullable(
                    erlang:element(8, Embed),
                    fun embed_img_to_json/1
                )},
            {<<"thumbnail"/utf8>>,
                gleam@json:nullable(
                    erlang:element(9, Embed),
                    fun embed_thumbnail_to_json/1
                )},
            {<<"video"/utf8>>,
                gleam@json:nullable(
                    erlang:element(10, Embed),
                    fun embed_video_to_json/1
                )},
            {<<"provider"/utf8>>,
                gleam@json:nullable(
                    erlang:element(11, Embed),
                    fun embed_provider_to_json/1
                )},
            {<<"author"/utf8>>,
                gleam@json:nullable(
                    erlang:element(12, Embed),
                    fun embed_author_to_json/1
                )},
            {<<"fields"/utf8>>,
                gleam@json:array(
                    gleam@option:unwrap(erlang:element(13, Embed), []),
                    fun embed_field_to_json/1
                )}]
    ).

-spec message_to_json(glyph@models@discord:message_payload()) -> binary().
message_to_json(Message) ->
    _pipe = gleam@json:object(
        [{<<"content"/utf8>>,
                gleam@json:nullable(
                    erlang:element(2, Message),
                    fun gleam@json:string/1
                )},
            {<<"tts"/utf8>>,
                gleam@json:nullable(
                    erlang:element(3, Message),
                    fun gleam@json:bool/1
                )},
            {<<"embeds"/utf8>>,
                gleam@json:array(
                    gleam@option:unwrap(erlang:element(4, Message), []),
                    fun embed_to_json/1
                )},
            {<<"allowed_mentions"/utf8>>,
                gleam@json:nullable(
                    erlang:element(5, Message),
                    fun allowed_mentions_to_json/1
                )},
            {<<"message_reference"/utf8>>,
                gleam@json:nullable(
                    erlang:element(6, Message),
                    fun message_reference_to_json/1
                )},
            {<<"sticker_ids"/utf8>>,
                gleam@json:array(
                    gleam@option:unwrap(erlang:element(7, Message), []),
                    fun gleam@json:string/1
                )},
            {<<"flags"/utf8>>,
                gleam@json:nullable(
                    erlang:element(8, Message),
                    fun gleam@json:int/1
                )},
            {<<"nonce"/utf8>>,
                gleam@json:nullable(
                    erlang:element(9, Message),
                    fun gleam@json:string/1
                )},
            {<<"enforce_nonce"/utf8>>,
                gleam@json:nullable(
                    erlang:element(10, Message),
                    fun gleam@json:bool/1
                )}]
    ),
    gleam@json:to_string(_pipe).
